#ifndef CAMLOAD_LIB_OBJC_H
#define CAMLOAD_LIB_OBJC_H
//
//  CamLoadLib_ObjC.h
//  CamLoadLib_ObjC
//
//  Created by Laurie Hufford on 6/15/10.
//  Copyright 2010 PhaseOne. All rights reserved.
//
#import <Cocoa/Cocoa.h>

#import "CamLoadLibEnums.h"
#import "DeviceServiceEnums.h"

//device property keys
extern NSString* const kDeviceIDKey;
extern NSString* const kDeviceTypeKey;
extern NSString* const kDeviceNameKey;
extern NSString* const kDeviceManufacturerKey;
//model property keys
extern NSString* const kModelIndexKey;
extern NSString* const kModelNameKey;
extern NSString* const kModelTypeKey;
extern NSString* const kModelSerialKey;
extern NSString* const kModelHWConfigKey;
extern NSString* const kModelIDKey;
extern NSString* const kModelCapCountKey;
extern NSString* const kModelHardwareRevisionKey;               // NSString eg. '1'

extern NSString* const kModelComponentNamesKey;                 // NSArray of NSStrings eg. ["TGEN", "BOOT"]
extern NSString* const kModelFirmwareVersionsKey;               // NSArray of NSStrings eg. ["1.3.213", "5.1.6"]
extern NSString* const kModelPackageVersionKey;                 // NSString eg.'1.3.234' or nil
extern NSString* const kModelSupportsPackageUploadKey;          // BOOL
extern NSString* const kModelDisconnectsAfterPackageUploadKey;  // BOOL
extern NSString* const kModelIsSerialNumberUpgradeRequiredKey;  // BOOL

extern NSString* const kModelMountKey;                          // NSString eg. 'M'
extern NSString* const kModelCalibDateKey;                      // NSNumber eg. 123456789 (secs from 1970)

extern NSString* const kCalibFileSerialKey;
extern NSString* const kCalibFileDateKey;

//notification keys
extern NSString* const kNotificationDeviceConnectedKey;
extern NSString* const kNotificationDeviceDisconnectedKey;
extern NSString* const kNotificationUpdateProgressKey;
extern NSString* const kNotificationUpdateSuccessfulKey;
extern NSString* const kNotificationUpdateFailureKey;
extern NSString* const kNotificationUpdateStoppedKey;
extern NSString* const kNotificationUpdateStatusUpdatedKey;
extern NSString* const kNotificationUpdateErrorMessageKey;
extern NSString* const kNotificationUpdateStartFailedKey;
extern NSString* const kNotificationUpdateStartedKey;
extern NSString* const kNotificationDeviceRebootRequestKey;
extern NSString* const kNotificationDeviceReconnectRequestKey;
extern NSString* const kNotificationDeviceModelDataChangedKey;

//event data keys
extern NSString* const kEventProgressInfoKey;
extern NSString* const kEventMessageKey;


typedef UInt32 DeviceId;
typedef NSUInteger ModelIndex;
typedef NSUInteger ComponentIndex;

@interface CamLoadLib_ObjC : NSObject { 
	void* _privateA; void* _privateB; id _privateC;
	
}
+(NSArray*) ComponentNamesArray;

-(NSDictionary*) GetDeviceProperties:(DeviceId)deviceID;
-(void)	RefreshDevicesList;
-(NSArray*) GetConnectedDeviceIDs;


-(void)	UpdateFirmwarePackage: (NSString*)filePath
                     deviceID: (DeviceId)deviceID
                    modelName: (NSString*)modelName;

-(void)	UpdateFirmwareComponent: (NSString*)filePath
                       deviceID: (DeviceId)deviceID
                      modelName: (NSString*)modelName
                  componentName: (NSString*)componentName;
/*
-(void)	UpdateFirmware: (NSString*)filePath 
			  deviceID: (DeviceId)deviceID
			modelIndex: (ModelIndex)modelIndex
		componentIndex: (ComponentIndex)componentIndex;
*/

-(NSUInteger) GetModelCount: (DeviceId)deviceID;

-(NSDictionary*) GetModelProperties: (DeviceId)deviceID
						 modelIndex: (ModelIndex)modelIndex;
-(NSDictionary*) GetModelProperties: (DeviceId)deviceID
						  modelName: (NSString*)modelName;

-(NSDictionary*) GetComponentVersions: (DeviceId)deviceID
						   modelIndex: (ModelIndex)modelIndex
						 userFriendly: (BOOL)userFriendly;

-(NSArray*) GetComponentNames: (DeviceId)deviceID 
					modelName: (NSString*)modelName;
-(NSArray*) GetComponentNames: (DeviceId)deviceID 
				   modelIndex: (ModelIndex)modelIndex;

-(NSArray*) GetFirmwareVersions: (DeviceId)deviceID
					  modelName: (NSString*)modelName
				   userFriendly: (BOOL)userFriendly;
-(NSArray*) GetFirmwareVersions: (DeviceId)deviceID
					 modelIndex: (ModelIndex)modelIndex
				   userFriendly: (BOOL)userFriendly;



-(BOOL) DoesDeviceSupportCalibrationFiles: (DeviceId)deviceID
							   modelIndex: (ModelIndex)modelIndex;

-(NSDictionary*) GetCalibrationFileInfo:(NSString*)calibFilePath deviceID: (DeviceId)deviceID;

-(BOOL) BackupCalibrationFileTo: (NSString*)backupPath
					   deviceID: (DeviceId)deviceID
					 modelIndex: (ModelIndex)modelIndex;

-(void) UploadCalibrationFile: (NSString*)calibFilePath
					 deviceID: (DeviceId)deviceID
				   modelIndex: (ModelIndex)modelIndex;


-(void) UpgradeSerialNumberTo:(NSString*)newSerialNumber
                     deviceID:(DeviceId)deviceID
                    modelName:(NSString*)modelName;
-(void) UpgradeSerialNumberTo:(NSString*)newSerialNumber
                     deviceID:(DeviceId)deviceID
                   modelIndex:(ModelIndex)modelIndex;

-(BOOL) IsSerialNumberValid:(NSString*)newSerialNumber
                   deviceID:(DeviceId)deviceID
                  modelName:(NSString*)modelName;
-(BOOL) IsSerialNumberValid:(NSString*)newSerialNumber
                   deviceID:(DeviceId)deviceID
                  modelIndex:(ModelIndex)modelIndex;


// Services
-(NSDictionary*) GetServiceIDsForDevice: (DeviceId)deviceID;
-(NSValue*) GetServiceWithID: (ServiceID) serviceID
            forDevice:(DeviceId)deviceID;

-(void) PerformAction:(ActionID)actioID
            ofService: (ServiceID) serviceID
           withParams:(NSValue*) params
            forDevice:(DeviceId)deviceID;



/*
-(NSComparisonResult) CompareFWComponentVersion: (DeviceId)deviceID
									  modelName: (NSString*)modelName
								  componentName: (NSString*)componentName
								  againstString: (NSString*)versionString;
-(NSComparisonResult) CompareFWComponentVersion: (DeviceId)deviceID
									 modelIndex: (ModelIndex)modelIndex
								 componentIndex: (ComponentIndex)componentIndex
								  againstString: (NSString*)versionString;

-(NSString*) FormatToUserFriendlyFWComponentVersion: (DeviceId)deviceID
										  modelName: (NSString*)modelName
									  componentName: (NSString*)componentName
									  versionString: (NSString*)versionString;
-(NSString*) FormatToUserFriendlyFWComponentVersion: (DeviceId)deviceID
										 modelIndex: (ModelIndex)modelIndex
									 componentIndex: (ComponentIndex)componentIndex
									  versionString: (NSString*)versionString;
*/
@end

#endif //CAMLOAD_LIB_OBJC_H
