//
//  ServiceCallParams.h
//  CamLoadLib
//
//  Created by Sergey Shramchenko on 09.07.13.
//  Copyright (c) 2013 PhaseOne. All rights reserved.
//

#ifndef CamLoadLib_ServiceCallParams_h
#define CamLoadLib_ServiceCallParams_h

#include "Typelist.h"
#include "TypeTraits.h"
#include "TypeManipAdditions.h"
#include "static_check.h"
#include <wchar.h>
#include <string>
#include <vector>



namespace P1 {
namespace CamLoad {
    namespace ServiceCall {
        typedef std::wstring          STRW;       // wide character string (16- or 32-bit)

        template<typename T>
        struct PureType {
            typedef typename ::Loki::TypeTraits<typename ::Loki::TypeTraits<T>::ParameterType>::ReferredType QualifiedType;
            typedef typename ::Loki::TypeTraits<QualifiedType>::UnqualifiedType Result;
        };
        
        typedef TYPELIST_1(STRW) KnownTypes;
        
        class ParamBase
        {
        public:
            typedef short TIndex;
       private:
            const void* m_Data;
            void* m_MutableData;
            TIndex m_TypeIndex;
        public:
            ParamBase(const void* theData, TIndex typeInadex)
            :m_Data(theData)
            ,m_MutableData(NULL)
            ,m_TypeIndex(typeInadex)
            {}
            ParamBase(void* theData, TIndex typeInadex)
            :m_Data(NULL)
            ,m_MutableData(theData)
            ,m_TypeIndex(typeInadex)
            {}
            const void* data() const
            {
                return (m_Data) ? m_Data : m_MutableData;
            }
            
            void* data()
            {
                return m_MutableData;
            }

            TIndex typeIndex() const
            {
                return m_TypeIndex;
            }
        };
        
        class Params {
        protected:
            mutable std::vector<ParamBase> m_Params;
            
        public:
            Params() {}
            Params(const ParamBase& param)
            {
                add(param);
            }
            void add(const ParamBase& param)
            {
                m_Params.push_back(param);
            }
            void clear() {
                m_Params.clear();
            }
            // For IN-parameters
            template <typename T>
            const typename PureType<T>::Result&  get_ref(ParamBase::TIndex idx) const;
            // For OUT-parameters
            template <typename T>
            typename PureType<T>::Result *
            get_ptr(ParamBase::TIndex idx) const;
        };
        
        struct TypeInformer {
            virtual short paramTypeAtIndex(ParamBase::TIndex) const = 0;
        };

        class TypeSafeParams : public Params {
        private:
            const TypeInformer* m_TypeInformer;
        public:
            TypeSafeParams(const TypeInformer* typeInformer)
            : m_TypeInformer(typeInformer)
            {}
            template <template<class> class T, class TParam>
            void addParam(const T<TParam>& param);            
        };
         
    }
}} // namescpace
    
#endif
